@@ -340,7 +340,7 @@ double *SLcomplex_tanh (double *tanhz, double *z)
    tanhz[1] = invden * sin (y);
    return tanhz;
 }
-
+#if 0
 static double *not_implemented (char *fun, double *p)
 {
    SLang_verror (SL_NOT_IMPLEMENTED, "%s for complex numbers has not been implemented",
@@ -348,20 +348,47 @@ static double *not_implemented (char *fun, double *p)
    *p = -1.0;
    return p;
 }
-
+#endif
+/* Use: asinh(z) = -i asin(iz) */
 double *SLcomplex_asinh (double *asinhz, double *z)
 {
-   return not_implemented ("asinh", asinhz);
+   double iz[2];
+   
+   iz[0] = -z[1];
+   iz[1] = z[0];
+   
+   (void) SLcomplex_asin (iz, iz);
+   asinhz[0] = iz[1];
+   asinhz[1] = -iz[0];
+   
+   return asinhz;
 }
 
+/* Use: acosh (z) = i acos(z) */
 double *SLcomplex_acosh (double *acoshz, double *z)
 {
-   return not_implemented ("acosh", acoshz);
+   double iz[2];
+   
+   (void) SLcomplex_acos (iz, z);
+   acoshz[0] = -iz[1];
+   acoshz[1] = iz[0];
+
+   return acoshz;
 }
 
+/* Use: atanh(z) = -i atan(iz) */
 double *SLcomplex_atanh (double *atanhz, double *z)
 {
-   return not_implemented ("atanh", atanhz);
+   double iz[2];
+   
+   iz[0] = -z[1];
+   iz[1] = z[0];
+   
+   (void) SLcomplex_atan (iz, iz);
+   atanhz[0] = iz[1];
+   atanhz[1] = -iz[0];
+   
+   return atanhz;
 }
 
 static int complex_binary_result (int op, unsigned char a, unsigned char b,
@@ -736,7 +763,7 @@ static int complex_generic_binary (int op,
       case SLANG_DIVIDE:	       /* / */
 	for (n = 0; n < n_max; n += 2)
 	  {
-	     int b0 = to_double((VOID_STAR)b);
+	     double b0 = to_double((VOID_STAR)b);
 	     if (b0 == 0)
 	       {
 		  SLang_Error = SL_DIVIDE_ERROR;
@@ -823,7 +850,7 @@ static int generic_complex_binary (int op,
       case SLANG_TIMES:
 	for (n = 0; n < n_max; n += 2)
 	  {
-	     int a0 = to_double((VOID_STAR)a);
+	     double a0 = to_double((VOID_STAR)a);
 	     c[n] = a0 * b[0];
 	     c[n + 1] = a0 * b[1];
 	     a += da; b += db;
