@@ -113,6 +113,7 @@ static int Has_HP_Glitch;
 #endif
 
 static char *Reset_Color_String;
+static int Is_Color_Terminal = 0;
 
 static int Linux_Console;
 
@@ -816,6 +817,18 @@ void SLtt_delete_nlines (int n)
 
 void SLtt_cls (void)
 {
+   /* If the terminal is a color terminal but the user wants black and 
+    * white, then make sure that the colors are reset.  This appears to be
+    * necessary.
+    */
+   if ((SLtt_Use_Ansi_Colors == 0) && Is_Color_Terminal)
+     {
+	if (Reset_Color_String != NULL)
+	  tt_write_string (Reset_Color_String);
+	else
+	  tt_write_string ("\033[0m\033[m");
+     }
+
    SLtt_normal_video();
    SLtt_reset_scroll_region ();
    tt_write_string(Cls_Str);
@@ -1888,6 +1901,9 @@ static void get_color_info (void)
    if (SLtt_Use_Ansi_Colors == 0)
      SLtt_Use_Ansi_Colors = (NULL != getenv ("COLORTERM"));
 
+   if (SLtt_Use_Ansi_Colors)
+     Is_Color_Terminal = 1;
+
 #if SLTT_HAS_NON_BCE_SUPPORT
    if (Can_Background_Color_Erase == 0)
      Can_Background_Color_Erase = (NULL != getenv ("COLORTERM_BCE"));
@@ -2051,6 +2067,7 @@ int SLtt_initialize (char *term)
 	  {
 	     int vt102 = 1;
 	     if (!strcmp (term, "vt100")) vt102 = 0;
+	     get_color_info ();
    	     SLtt_set_term_vtxxx (&vt102);
 	     return 0;
 	  }
@@ -2263,6 +2280,7 @@ int SLtt_initialize (char *term)
 #endif
    get_color_info ();
 
+  
    if ((Cls_Str == NULL)
        || (Curs_Pos_Str == NULL))
      return -2;
