@@ -293,7 +293,7 @@ unsigned int _SLsys_getkey (void)
 	/* drop */
       case 0xe0:
       case 0:			/* extended key code */
-	ch = _SLpc_convert_scancode (scan);
+	ch = _SLpc_convert_scancode (scan, 0, 1);
      }
    return (ch);
 }
