@@ -746,6 +746,14 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	SLkm_define_key  ("^@S", (FVOID_STAR) rl_del, RL_Keymap);
 	SLkm_define_key  ("^@O", (FVOID_STAR) rl_eol, RL_Keymap);
 	SLkm_define_key  ("^@G", (FVOID_STAR) rl_bol, RL_Keymap);
+
+	SLkm_define_key  ("\xE0H", (FVOID_STAR) rl_prev_line, RL_Keymap);
+	SLkm_define_key  ("\xE0P", (FVOID_STAR) rl_next_line, RL_Keymap);
+	SLkm_define_key  ("\xE0M", (FVOID_STAR) rl_right, RL_Keymap);
+	SLkm_define_key  ("\xE0K", (FVOID_STAR) rl_left, RL_Keymap);
+	SLkm_define_key  ("\xE0S", (FVOID_STAR) rl_del, RL_Keymap);
+	SLkm_define_key  ("\xE0O", (FVOID_STAR) rl_eol, RL_Keymap);
+	SLkm_define_key  ("\xE0G", (FVOID_STAR) rl_bol, RL_Keymap);
 #endif
 	SLkm_define_key  ("^C", (FVOID_STAR) rl_abort, RL_Keymap);
 	SLkm_define_key  ("^E", (FVOID_STAR) rl_eol, RL_Keymap);
