@@ -15,7 +15,7 @@ typedef struct Screen_Type
   {
      int n;                    /* number of chars written last time */
      int flags;                /* line untouched, etc... */
-     unsigned short *old, *neew;
+     SLsmg_Char_Type *old, *neew;
 #ifndef IBMPC_SYSTEM
      unsigned long old_hash, new_hash;
 #endif
@@ -63,7 +63,7 @@ static void (*tt_normal_video)(void) = SLtt_normal_video;
 static void (*tt_goto_rc)(int, int) = SLtt_goto_rc;
 static void (*tt_cls) (void) = SLtt_cls;
 static void (*tt_del_eol) (void) = SLtt_del_eol;
-static void (*tt_smart_puts) (unsigned short *, unsigned short *, int, int) = SLtt_smart_puts;
+static void (*tt_smart_puts) (SLsmg_Char_Type *, SLsmg_Char_Type *, int, int) = SLtt_smart_puts;
 static int (*tt_flush_output) (void) = SLtt_flush_output;
 static int (*tt_reset_video) (void) = SLtt_reset_video;
 static int (*tt_init_video) (void) = SLtt_init_video;
@@ -86,12 +86,12 @@ static int *tt_Use_Blink_For_ACS = &SLtt_Use_Blink_For_ACS;
 
 static int Smg_Inited;
 
-static void blank_line (unsigned short *p, int n, unsigned char ch)
+static void blank_line (SLsmg_Char_Type *p, int n, unsigned char ch)
 {
-   register unsigned short *pmax = p + n;
-   register unsigned short color_ch;
+   register SLsmg_Char_Type *pmax = p + n;
+   register SLsmg_Char_Type color_ch;
 
-   color_ch = (This_Color << 8) | (unsigned short) ch;
+   color_ch = SLSMG_BUILD_CHAR(ch,This_Color);
 
    while (p < pmax)
      {
@@ -133,7 +133,7 @@ void SLsmg_erase_eol (void)
 static void scroll_up (void)
 {
    unsigned int i, imax;
-   unsigned short *neew;
+   SLsmg_Char_Type *neew;
 
    neew = SL_Screen[0].neew;
    imax = Screen_Rows - 1;
@@ -300,7 +300,7 @@ int SLsmg_Display_Eight_Bit = 128;
 
 void SLsmg_write_nchars (char *str, unsigned int n)
 {
-   register unsigned short *p, old, neew, color;
+   register SLsmg_Char_Type *p, old, neew, color;
    unsigned char ch;
    unsigned int flags;
    int len, start_len, max_len;
@@ -317,7 +317,7 @@ void SLsmg_write_nchars (char *str, unsigned int n)
    if (Smg_Inited == 0) return;
 
    str_max = str + n;
-   color = This_Color << 8;
+   color = This_Color;
 
    top:				       /* get here only on newline */
 
@@ -352,7 +352,7 @@ void SLsmg_write_nchars (char *str, unsigned int n)
 	     if (len > start_len)
 	       {
 		  old = *p;
-		  neew = color | (unsigned short) ch;
+		  neew = SLSMG_BUILD_CHAR(ch,color);
 		  if (old != neew)
 		    {
 		       flags |= TOUCHED;
@@ -369,7 +369,7 @@ void SLsmg_write_nchars (char *str, unsigned int n)
 	     n = SLsmg_Tab_Width - (n % SLsmg_Tab_Width);
 	     if ((unsigned int) len + n > (unsigned int) max_len)
 	       n = (unsigned int) (max_len - len);
-	     neew = color | (unsigned short) ' ';
+	     neew = SLSMG_BUILD_CHAR(' ',color);
 	     while (n--)
 	       {
 		  len += 1;
@@ -398,7 +398,7 @@ void SLsmg_write_nchars (char *str, unsigned int n)
 	  {
 	     if (ch & 0x80)
 	       {
-		  neew = color | (unsigned short) '~';
+		  neew = SLSMG_BUILD_CHAR('~',color);
 		  len += 1;
 		  if (len > start_len)
 		    {
@@ -416,7 +416,7 @@ void SLsmg_write_nchars (char *str, unsigned int n)
 	     len += 1;
 	     if (len > start_len)
 	       {
-		  neew = color | (unsigned short) '^';
+		  neew = SLSMG_BUILD_CHAR('^',color);
 		  if (*p != neew)
 		    {
 		       *p = neew;
@@ -430,7 +430,7 @@ void SLsmg_write_nchars (char *str, unsigned int n)
 	     len++;
 	     if (len > start_len)
 	       {
-		  neew = color | (unsigned short) ch;
+		  neew = SLSMG_BUILD_CHAR(ch,color);
 		  if (*p != neew)
 		    {
 		       *p = neew;
@@ -487,9 +487,9 @@ void SLsmg_cls (void)
    Cls_Flag = 1;
 }
 #if 0
-static void do_copy (unsigned short *a, unsigned short *b)
+static void do_copy (SLsmg_Char_Type *a, SLsmg_Char_Type *b)
 {
-   unsigned short *amax = a + Screen_Cols;
+   SLsmg_Char_Type *amax = a + Screen_Cols;
 
    while (a < amax) *a++ = *b++;
 }
@@ -497,11 +497,11 @@ static void do_copy (unsigned short *a, unsigned short *b)
 
 #ifndef IBMPC_SYSTEM
 int SLsmg_Scroll_Hash_Border = 0;
-static unsigned long compute_hash (unsigned short *s, int n)
+static unsigned long compute_hash (SLsmg_Char_Type *s, int n)
 {
    register unsigned long h = 0, g;
    register unsigned long sum = 0;
-   register unsigned short *smax, ch;
+   register SLsmg_Char_Type *smax, ch;
    int is_blank = 2;
 
    s += SLsmg_Scroll_Hash_Border;
@@ -509,7 +509,7 @@ static unsigned long compute_hash (unsigned short *s, int n)
    while (s < smax)
      {
 	ch = *s++;
-	if (is_blank && ((ch & 0xFF) != 32)) is_blank--;
+	if (is_blank && (SLSMG_EXTRACT_CHAR(ch) != 32)) is_blank--;
 
 	sum += ch;
 
@@ -532,7 +532,7 @@ static int try_scroll_down (int rmin, int rmax)
    unsigned long hash;
    int did_scroll;
    int color;
-   unsigned short *tmp;
+   SLsmg_Char_Type *tmp;
    int ignore;
 
    did_scroll = 0;
@@ -630,7 +630,7 @@ static int try_scroll_up (int rmin, int rmax)
    unsigned long hash;
    int did_scroll;
    int color;
-   unsigned short *tmp;
+   SLsmg_Char_Type *tmp;
    int ignore;
 
    did_scroll = 0;
@@ -802,7 +802,7 @@ static void adjust_colors (void)
 
    for (i = 0; i < Screen_Rows; i++)
      {
-	unsigned short *s, *smax;
+	SLsmg_Char_Type *s, *smax;
 
 	SL_Screen[i].flags |= TRASHED;
 	s = SL_Screen[i].neew;
@@ -810,7 +810,7 @@ static void adjust_colors (void)
 	
 	while (s < smax)
 	  {
-	     int color = (int) (*s >> 8);
+	     int color = (int) SLSMG_EXTRACT_COLOR(*s);
 	     int acs;
 
 	     if (color < 0)
@@ -823,7 +823,10 @@ static void adjust_colors (void)
 	     color = (color & 0x7F) - bce;
 	     color += Bce_Color_Offset;
 	     if (color >= 0)
-	       *s = (*s & 0xFF) | (((color&0x7F)|acs) << 8);
+	       {
+		  unsigned char ch = SLSMG_EXTRACT_CHAR(*s);
+		  *s = SLSMG_BUILD_CHAR(ch, ((color&0x7F)|acs));
+	       }
 	     s++;
 	  }
      }
@@ -886,7 +889,7 @@ void SLsmg_refresh (void)
 	(*tt_smart_puts) (SL_Screen[i].neew, SL_Screen[i].old, Screen_Cols, i);
 
 	SLMEMCPY ((char *) SL_Screen[i].old, (char *) SL_Screen[i].neew,
-		  Screen_Cols * sizeof (short));
+		  Screen_Cols * sizeof (SLsmg_Char_Type));
 
 	SL_Screen[i].flags = 0;
 #ifndef IBMPC_SYSTEM
@@ -1056,7 +1059,7 @@ static void reset_smg (void)
 static int init_smg (void)
 {
    int i, len;
-   unsigned short *old, *neew;
+   SLsmg_Char_Type *old, *neew;
 
    Smg_Inited = 0;
 
@@ -1081,8 +1084,8 @@ static int init_smg (void)
    len = Screen_Cols + 3;
    for (i = 0; i < Screen_Rows; i++)
      {
-	if ((NULL == (old = (unsigned short *) SLmalloc (sizeof(short) * len)))
-	    || ((NULL == (neew = (unsigned short *) SLmalloc (sizeof(short) * len)))))
+	if ((NULL == (old = (SLsmg_Char_Type *) SLmalloc (sizeof(SLsmg_Char_Type) * len)))
+	    || ((NULL == (neew = (SLsmg_Char_Type *) SLmalloc (sizeof(SLsmg_Char_Type) * len)))))
 	  {
 	     SLfree ((char *) old);
 	     return -1;
@@ -1153,7 +1156,7 @@ void SLsmg_reset_smg (void)
    UNBLOCK_SIGNALS;
 }
 
-unsigned short SLsmg_char_at (void)
+SLsmg_Char_Type SLsmg_char_at (void)
 {
    if (Smg_Inited == 0) return 0;
 
@@ -1366,9 +1369,9 @@ void SLsmg_forward (int n)
    This_Col += n;
 }
 
-void SLsmg_write_color_chars (unsigned short *s, unsigned int len)
+void SLsmg_write_color_chars (SLsmg_Char_Type *s, unsigned int len)
 {
-   unsigned short *smax, sh;
+   SLsmg_Char_Type *smax, sh;
    char buf[32], *b, *bmax;
    int color, save_color;
 
@@ -1384,7 +1387,7 @@ void SLsmg_write_color_chars (unsigned short *s, unsigned int len)
      {
 	sh = *s++;
 
-	color = sh >> 8;
+	color = SLSMG_EXTRACT_COLOR(sh);
 
 #if REQUIRES_NON_BCE_SUPPORT
 	if (Bce_Color_Offset)
@@ -1405,7 +1408,7 @@ void SLsmg_write_color_chars (unsigned short *s, unsigned int len)
 	       }
 	     This_Color = color;
 	  }
-	*b++ = (char) (sh & 0xFF);
+	*b++ = (char) SLSMG_EXTRACT_CHAR(sh);
      }
 
    if (b != buf)
@@ -1414,7 +1417,7 @@ void SLsmg_write_color_chars (unsigned short *s, unsigned int len)
    This_Color = save_color;
 }
 
-unsigned int SLsmg_read_raw (unsigned short *buf, unsigned int len)
+unsigned int SLsmg_read_raw (SLsmg_Char_Type *buf, unsigned int len)
 {
    unsigned int r, c;
 
@@ -1428,14 +1431,14 @@ unsigned int SLsmg_read_raw (unsigned short *buf, unsigned int len)
    if (c + len > (unsigned int) Screen_Cols)
      len = (unsigned int) Screen_Cols - c;
 
-   memcpy ((char *) buf, (char *) (SL_Screen[r].neew + c), len * sizeof (short));
+   memcpy ((char *) buf, (char *) (SL_Screen[r].neew + c), len * sizeof (SLsmg_Char_Type));
    return len;
 }
 
-unsigned int SLsmg_write_raw (unsigned short *buf, unsigned int len)
+unsigned int SLsmg_write_raw (SLsmg_Char_Type *buf, unsigned int len)
 {
    unsigned int r, c;
-   unsigned short *dest;
+   SLsmg_Char_Type *dest;
 
    if (Smg_Inited == 0) return 0;
 
@@ -1449,9 +1452,9 @@ unsigned int SLsmg_write_raw (unsigned short *buf, unsigned int len)
 
    dest = SL_Screen[r].neew + c;
 
-   if (0 != memcmp ((char *) dest, (char *) buf, len * sizeof (short)))
+   if (0 != memcmp ((char *) dest, (char *) buf, len * sizeof (SLsmg_Char_Type)))
      {
-	memcpy ((char *) dest, (char *) buf, len * sizeof (short));
+	memcpy ((char *) dest, (char *) buf, len * sizeof (SLsmg_Char_Type));
 	SL_Screen[r].flags |= TOUCHED;
      }
    return len;
@@ -1461,7 +1464,7 @@ void
 SLsmg_set_color_in_region (int color, int r, int c, unsigned int dr, unsigned int dc)
 {
    int cmax, rmax;
-   unsigned short char_mask;
+   SLsmg_Char_Type char_mask;
 
    if (Smg_Inited == 0) return;
 
@@ -1477,6 +1480,15 @@ SLsmg_set_color_in_region (int color, int r, int c, unsigned int dr, unsigned in
    if (c < 0) c = 0;
    if (r < 0) r = 0;
 
+#if REQUIRES_NON_BCE_SUPPORT
+   if (Bce_Color_Offset)
+     {
+	if (color & 0x80)
+	  color = ((color & 0x7F) + Bce_Color_Offset) | 0x80;
+	else
+	  color = ((color & 0x7F) + Bce_Color_Offset) & 0x7F;
+     }
+#endif
    color = color << 8;
 
    char_mask = 0xFF;
@@ -1489,7 +1501,7 @@ SLsmg_set_color_in_region (int color, int r, int c, unsigned int dr, unsigned in
 
    while (r < rmax)
      {
-	unsigned short *s, *smax;
+	SLsmg_Char_Type *s, *smax;
 
 	SL_Screen[r].flags |= TOUCHED;
 	s = SL_Screen[r].neew;
